@@ -146,13 +146,14 @@ class APIAuthenticatorBase:
         category=SingerSDKDeprecationWarning,
     )
     def tap_name(self) -> str:
-        """Get tap name.
-
-        Returns:
-            The tap name.
-        """
+        """The tap name."""
         return self._tap_name  # pragma: no cover
 
+    @tap_name.setter
+    def tap_name(self, name: str) -> None:
+        """Set the tap name."""
+        self._tap_name = name
+
     @property
     @deprecated(
         "The `config` property is deprecated and will be removed in a future version",
@@ -635,6 +636,7 @@ class OAuthAuthenticator(APIAuthenticatorBase):
         Raises:
             RuntimeError: When OAuth login fails.
         """
+        self.logger.info("Requesting new access token")
         request_time = utc_now()
         auth_request_payload = self.oauth_request_payload
         token_response = requests.post(
@@ -649,7 +651,7 @@ class OAuthAuthenticator(APIAuthenticatorBase):
             msg = f"Failed OAuth login, response was '{token_response.json()}'. {ex}"
             raise RuntimeError(msg) from ex
 
-        self.logger.info("OAuth authorization attempt was successful.")
+        self.logger.debug("OAuth authorization attempt was successful")
 
         token_json = token_response.json()
         self.access_token = token_json["access_token"]
