@@ -193,13 +193,8 @@ class Sink(metaclass=abc.ABCMeta):  # noqa: PLR0904
         self._warned_missing_fields: set[str] = set()
 
         self._validator: BaseJSONSchemaValidator | None = self.get_validator()
-        self._record_counter: metrics.Counter = metrics.record_counter(self.stream_name)
-        self._batch_timer = metrics.Timer(
-            metrics.Metric.BATCH_PROCESSING_TIME,
-            tags={
-                metrics.Tag.STREAM: self.stream_name,
-            },
-        )
+        self._record_counter: metrics.Counter = self.get_sink_record_counter()
+        self._batch_timer = self.get_batch_processing_timer()
 
     @property
     def record_counter_metric(self) -> metrics.Counter:
@@ -219,6 +214,37 @@ class Sink(metaclass=abc.ABCMeta):  # noqa: PLR0904
         """
         return self._batch_timer
 
+    def get_sink_record_counter(self) -> metrics.Counter:
+        """Get a counter for counting records processed by this sink.
+
+        This method can be overridden to customize the record counter, for example
+        to add custom tags or modify the logging behavior.
+
+        Returns:
+            A counter for counting records.
+
+        .. versionadded:: 0.51.0
+        """
+        return metrics.record_counter(self.stream_name)
+
+    def get_batch_processing_timer(self) -> metrics.Timer:
+        """Get a timer for measuring batch processing time.
+
+        This method can be overridden to customize the batch processing timer,
+        for example to add custom tags or modify the logging behavior.
+
+        Returns:
+            A timer for measuring batch processing time.
+
+        .. versionadded:: 0.51.0
+        """
+        return metrics.Timer(
+            metrics.Metric.BATCH_PROCESSING_TIME,
+            tags={
+                metrics.Tag.STREAM: self.stream_name,
+            },
+        )
+
     @cached_property
     def validate_schema(self) -> bool:
         """Enable JSON schema record validation.
@@ -226,7 +252,7 @@ class Sink(metaclass=abc.ABCMeta):  # noqa: PLR0904
         Returns:
             True if JSON schema validation is enabled.
         """
-        return self.config.get("validate_records", True)
+        return self.config.get("validate_records", True)  # type: ignore[no-any-return]
 
     def get_validator(self) -> BaseJSONSchemaValidator | None:
         """Get a record validator for this sink.
@@ -396,7 +422,7 @@ class Sink(metaclass=abc.ABCMeta):  # noqa: PLR0904
         Returns:
             True if metadata columns should be added.
         """
-        return self.config.get("add_record_metadata", False)
+        return self.config.get("add_record_metadata", False)  # type: ignore[no-any-return]
 
     @property
     def process_activate_version_messages(self) -> bool:
@@ -405,7 +431,7 @@ class Sink(metaclass=abc.ABCMeta):  # noqa: PLR0904
         Returns:
             True if activate version messages should be processed.
         """
-        return self.config.get("process_activate_version_messages", True)
+        return self.config.get("process_activate_version_messages", True)  # type: ignore[no-any-return]
 
     @property
     def datetime_error_treatment(self) -> DatetimeErrorTreatmentEnum:
@@ -734,6 +760,9 @@ class Sink(metaclass=abc.ABCMeta):  # noqa: PLR0904
     ) -> None:
         """Process a batch file with the given batch context.
 
+        For JSONL-encoded batch files, expects raw JSON records (one per line),
+        not Singer protocol messages.
+
         Args:
             encoding: The batch file encoding.
             files: The batch files to process.
