@@ -162,7 +162,6 @@ class SQLToJSONSchema:
 
         Args:
             config: The configuration dictionary.
-            use_singer_decimal: Whether to represent numbers as `string` with
                 the `x-singer.decimal` format instead of as `number`.
 
         Returns:
@@ -605,6 +604,7 @@ class SQLConnector:  # noqa: PLR0904
         """
         self._config: dict[str, t.Any] = config or {}
         self._sqlalchemy_url: str | None = sqlalchemy_url or None
+        self._tables_prepared: dict[str, bool] = {}
 
     @property
     def config(self) -> dict:
@@ -1011,7 +1011,6 @@ class SQLConnector:  # noqa: PLR0904
             schema_name: Schema name to inspect
             table_name: Name of the table or a view
             is_view: Flag whether this object is a view, returned by `get_object_names`
-            reflect_indices: Whether to reflect indices
             reflected_columns: List of reflected columns
             reflected_pk: Reflected primary key
             reflected_indices: List of reflected indices
@@ -1396,17 +1395,24 @@ class SQLConnector:  # noqa: PLR0904
             partition_keys: list of partition keys.
             as_temp_table: True to create a temp table.
         """
-        if not self.table_exists(full_table_name=full_table_name):
-            self.create_empty_table(
-                full_table_name=full_table_name,
-                schema=schema,
-                primary_keys=primary_keys,
-                partition_keys=partition_keys,
-                as_temp_table=as_temp_table,
+        table_key = str(full_table_name)
+        table_already_prepared = self._tables_prepared.get(table_key, False)
+
+        # Only drop/recreate on OVERWRITE if this is the first time preparing the table
+        # in this target run. Subsequent schema changes should add columns
+        # incrementally.
+        if (
+            (
+                self.config["load_method"] == TargetLoadMethods.OVERWRITE
+                and not table_already_prepared
             )
-            return
-        if self.config["load_method"] == TargetLoadMethods.OVERWRITE:
-            self.get_table(full_table_name=full_table_name).drop(self._engine)
+            or not self.table_exists(full_table_name=full_table_name)  # First-time sync
+        ):
+            _, schema_name, table_name = self.parse_full_table_name(full_table_name)
+            meta = sa.MetaData()
+            table = sa.Table(table_name, meta, schema=schema_name)
+            table.drop(self._engine, checkfirst=True)
+            self.logger.info("Creating empty table %s", full_table_name)
             self.create_empty_table(
                 full_table_name=full_table_name,
                 schema=schema,
@@ -1414,8 +1420,11 @@ class SQLConnector:  # noqa: PLR0904
                 partition_keys=partition_keys,
                 as_temp_table=as_temp_table,
             )
+            self._tables_prepared[table_key] = True
             return
 
+        # For non-OVERWRITE modes or subsequent schema changes, add columns
+        # incrementally
         for property_name, property_def in schema["properties"].items():
             self.prepare_column(
                 full_table_name,
@@ -1428,6 +1437,8 @@ class SQLConnector:  # noqa: PLR0904
             primary_keys=primary_keys,
         )
 
+        self._tables_prepared[table_key] = True
+
     def prepare_primary_key(
         self,
         *,
