@@ -38,9 +38,13 @@ from singer_sdk.typing import (
     DurationType,
     EmailType,
     HostnameType,
+    IDNEmailType,
+    IDNHostnameType,
     IntegerType,
     IPv4Type,
     IPv6Type,
+    IRIReferenceType,
+    IRIType,
     JSONPointerType,
     ObjectType,
     PropertiesList,
@@ -304,6 +308,13 @@ def test_property_title():
                 "format": "email",
             },
         ),
+        (
+            IDNEmailType,
+            {
+                "type": ["string"],
+                "format": "idn-email",
+            },
+        ),
         (
             HostnameType,
             {
@@ -311,6 +322,13 @@ def test_property_title():
                 "format": "hostname",
             },
         ),
+        (
+            IDNHostnameType,
+            {
+                "type": ["string"],
+                "format": "idn-hostname",
+            },
+        ),
         (
             IPv4Type,
             {
@@ -339,6 +357,13 @@ def test_property_title():
                 "format": "uri",
             },
         ),
+        (
+            IRIType,
+            {
+                "type": ["string"],
+                "format": "iri",
+            },
+        ),
         (
             URIReferenceType,
             {
@@ -346,6 +371,13 @@ def test_property_title():
                 "format": "uri-reference",
             },
         ),
+        (
+            IRIReferenceType,
+            {
+                "type": ["string"],
+                "format": "iri-reference",
+            },
+        ),
         (
             URITemplateType,
             {
